@@ -20,6 +20,7 @@ import fixtures as og_fixtures
 from neutron_lib.api.definitions import allowedaddresspairs
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import provider_net
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from oslo_concurrency import processutils
@@ -60,10 +61,10 @@ class WaitForDataPathBindingCreateEvent(event.WaitEvent):
 class WaitForChassisPrivateCreateEvent(event.WaitEvent):
     event_name = 'WaitForChassisPrivateCreateEvent'
 
-    def __init__(self, chassis_name, chassis_table):
+    def __init__(self, chassis_name):
         events = (self.ROW_CREATE,)
         conditions = (('name', '=', chassis_name),)
-        super().__init__(events, chassis_table, conditions, timeout=15)
+        super().__init__(events, 'Chassis_Private', conditions, timeout=15)
 
 
 class DistributedLockTestEvent(event.WaitEvent):
@@ -357,8 +358,8 @@ class TestNBDbMonitor(testlib_api.MySQLTestCaseMixin,
 
             # Since the virtual parents is a string, representing a list of
             # ports, we should make a set() and compare sets
-            bp_set = {p for p in vp.split(',')}
-            vp_set = {p for p in vparents.split(',')}
+            bp_set = set(vp.split(','))
+            vp_set = set(vparents.split(','))
             return bp_set == vp_set
 
         check = functools.partial(is_port_virtual_parents, port_id, vparents)
@@ -474,7 +475,7 @@ class TestNBDbMonitor(testlib_api.MySQLTestCaseMixin,
                 ovn_port_binding_revision_number)
 
         check = functools.partial(
-            is_port_and_port_binding_same_revision_number, port_id)
+            is_port_and_port_binding_same_revision_number,port_id)
         n_utils.wait_until_true(check, timeout=10)
 
     def test_virtual_port_host_update_upon_failover(self):
@@ -552,8 +553,10 @@ class TestSBDbMonitor(base.TestOVNFunctionalBase, test_l3.L3NatTestCaseMixin):
 
     def setUp(self, **kwargs):
         super().setUp(**kwargs)
-        self.chassis = self.add_fake_chassis('ovs-host1',
-                                             enable_chassis_as_gw=True)
+        self.physnet = 'public'
+        self.chassis = self.add_fake_chassis(
+            'ovs-host1', physical_nets=[self.physnet],
+            enable_chassis_as_gw=True)
         self.l3_plugin = directory.get_plugin(plugin_constants.L3)
         ext_mgr = test_l3.L3TestExtensionManager()
         self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
@@ -580,10 +583,12 @@ class TestSBDbMonitor(base.TestOVNFunctionalBase, test_l3.L3NatTestCaseMixin):
                 return True
             return lrp.external_ids == pb.external_ids
 
-        kwargs = {'arg_list': (external_net.EXTERNAL,),
-                  external_net.EXTERNAL: True}
+        arg_dict = {external_net.EXTERNAL: True,
+                    provider_net.NETWORK_TYPE: 'flat',
+                    provider_net.PHYSICAL_NETWORK: self.physnet}
         ext_net = self._make_network(self.fmt, 'ext_net', True, as_admin=True,
-                                     **kwargs)
+                                     arg_list=tuple(arg_dict.keys()),
+                                     **arg_dict)
         self._make_subnet(self.fmt, ext_net, '10.251.0.1', '10.251.0.0/24',
                           enable_dhcp=True)
         router = self._make_router(self.fmt, self._tenant_id)
@@ -617,8 +622,7 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
         self.handler = self.sb_api.idl.notify_handler
         self.mock_ovsdb_idl = mock.Mock()
         chassis_name = uuidutils.generate_uuid()
-        row_event = WaitForChassisPrivateCreateEvent(
-            chassis_name, self.mech_driver.agent_chassis_table)
+        row_event = WaitForChassisPrivateCreateEvent(chassis_name)
         self.mech_driver.sb_ovn.idl.notify_handler.watch_event(row_event)
         self.chassis_name = self.add_fake_chassis(
             self.FAKE_CHASSIS_HOST, name=chassis_name,
@@ -628,15 +632,22 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
             lambda: len(list(neutron_agent.AgentCache())) == 1)
 
     def test_agent_change_controller(self):
-        self.assertEqual(neutron_agent.ControllerGatewayAgent,
-                type(neutron_agent.AgentCache().get(self.chassis_name)))
+        self.assertIsInstance(
+            neutron_agent.AgentCache().get(self.chassis_name),
+            neutron_agent.ControllerGatewayAgent)
         self.sb_api.db_set('Chassis', self.chassis_name, ('other_config',
-                {'ovn-cms-options': ''})).execute(check_error=True)
+            {'ovn-cms-options': ''})).execute(check_error=True)
         n_utils.wait_until_true(lambda:
-                neutron_agent.AgentCache().get(self.chassis_name).
-                chassis.other_config['ovn-cms-options'] == '')
-        self.assertEqual(neutron_agent.ControllerAgent,
-                type(neutron_agent.AgentCache().get(self.chassis_name)))
+            isinstance(neutron_agent.AgentCache().get(self.chassis_name),
+                       neutron_agent.ControllerAgent))
+
+        # Change back to gw chassis
+        self.sb_api.db_set('Chassis', self.chassis_name, ('other_config',
+            {'ovn-cms-options': 'enable-chassis-as-gw'})).execute(
+                check_error=True)
+        n_utils.wait_until_true(lambda:
+            isinstance(neutron_agent.AgentCache().get(self.chassis_name),
+                       neutron_agent.ControllerGatewayAgent))
 
     def test_agent_updated_at_use_nb_cfg_timestamp(self):
         def check_agent_ts():
@@ -648,8 +659,6 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
                 int(chassis_ts / 1000), datetime.timezone.utc)
             return agent.updated_at == updated_at
 
-        if not self.sb_api.is_table_present('Chassis_Private'):
-            self.skipTest('Ovn sb not support Chassis_Private')
         timestamp = timeutils.utcnow_ts()
         nb_cfg_timestamp = timestamp * 1000
         self.sb_api.db_set('Chassis_Private', self.chassis_name, (
@@ -682,9 +691,6 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
             agent = neutron_agent.AgentCache().get(self.chassis_name)
             return agent.updated_at != 0
 
-        if not self.sb_api.is_table_present('Chassis_Private'):
-            self.skipTest('Ovn sb not support Chassis_Private')
-
         # Set nb_cfg to some realistic value, so that the alive check can
         # actually work
         self.nb_api.db_set(
